/*
 * Copyright (c) 2019-present Sonatype, Inc. All rights reserved.
 * Includes the third-party code listed at http://links.sonatype.com/products/nexus/attributions.
 * "Sonatype" is a trademark of Sonatype, Inc.
 */
package com.sonatype.iq.artifactory

import com.sonatype.iq.artifactory.safeMode.SafeModeNexusFirewallForArtifactory

import groovy.json.JsonOutput
import groovy.transform.Field
import org.artifactory.fs.ItemInfo
import org.artifactory.repo.RepoPath
import org.artifactory.repo.Repositories
import org.artifactory.request.Request

@Field NexusFirewallForArtifactory nexusFirewallForArtifactory

initPlugin(repositories)

executions {
  /**
   * This will return a summary fo the Firewall policy evaluation for a particular repository
   *
   * This can be triggered with the following curl command:
   * curl -u admin "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/firewallEvaluationSummary?params=repo=maven-central"
   **/
  firewallEvaluationSummary(httpMethod: 'GET', groups: ['readers']) { Map<String, List<String>> params ->
    if (!params.repo || params.repo.size() != 1) {
      status = 500
      message = 'Must supply a single repo name as a query parameter, i.e. ?params=repo=maven-central'
      return
    }

    try {
      message = JsonOutput.toJson(this.nexusFirewallForArtifactory.getFirewallEvaluationSummary(params.repo.get(0)))
    } catch (e) {
      status = 500
      message = e.message
    }
  }

  /**
   * Return the plugin version
   *
   * curl -u admin "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/firewallVersion"
   **/
  firewallVersion(httpMethod: 'GET', groups: ['readers']) { Map<String, List<String>> params ->
    try {
      message = JsonOutput.toJson([version: nexusFirewallForArtifactory.getPluginVersion()])
    } catch (e) {
      message = e.message
    }
  }
}

jobs {
  // reload at noon every day
  loadIgnorePatterns(cron: nexusFirewallForArtifactory.getIgnorePatternReloadCronExpression()) {
    nexusFirewallForArtifactory.loadIgnorePatterns()
  }

  //interval here is not important, this will only run once on startup
  scan(delay: 60000, interval: 86400000) {
    nexusFirewallForArtifactory.auditRepositories()
  }

  //interval here is not important, this will only run once immediately on startup
  initRepositories(delay: 1, interval:86400000){
    nexusFirewallForArtifactory.verifyInit()
  }
}


download {
  beforeDownload { Request request, RepoPath repoPath ->
    this.nexusFirewallForArtifactory.beforeDownloadHandler(repoPath)
  }
}

storage {
  beforePropertyCreate { ItemInfo item, String name, String[] values ->
    nexusFirewallForArtifactory.propertyEventHandler(name)
  }
  beforePropertyDelete { ItemInfo item, String name ->
    nexusFirewallForArtifactory.propertyEventHandler(name)
  }
}

private void initPlugin(Repositories repositories) {
  try {
    nexusFirewallForArtifactory = new DefaultNexusFirewallForArtifactory(log, ctx.artifactoryHome.pluginsDir,
        repositories, searches, ctx.versionProvider?.running?.versionName)
  }
  catch (MissingPropertiesException e) {
    nexusFirewallForArtifactory = new SafeModeNexusFirewallForArtifactory(log)
  }
  catch (e) {
    log.error(e.message)
    throw e
  }
  nexusFirewallForArtifactory.init()
}
